library benchmarks.src.costs.index;

import "package:angular2/bootstrap.dart" show bootstrap;
import "package:angular2/core.dart"
    show Component, Directive, DynamicComponentLoader, ElementRef;
import "package:angular2/common.dart" show NgIf, NgFor;
import "package:angular2/src/core/application_ref.dart" show ApplicationRef;
import "package:angular2/src/facade/collection.dart" show ListWrapper;
import "package:angular2/src/testing/benchmark_util.dart"
    show getIntParameter, bindAction;

var testList = null;
main() {
  var size = getIntParameter("size");
  testList = ListWrapper.createFixedSize(size);
  bootstrap(AppComponent).then((ref) {
    var injector = ref.injector;
    AppComponent app = ref.hostComponent;
    var appRef = injector.get(ApplicationRef);
    bindAction("#reset", () {
      app.reset();
      appRef.tick();
    });
    // Baseline (plain components)
    bindAction("#createPlainComponents", () {
      app.createPlainComponents();
      appRef.tick();
    });
    // Components with decorators
    bindAction("#createComponentsWithDirectives", () {
      app.createComponentsWithDirectives();
      appRef.tick();
    });
    // Components with decorators
    bindAction("#createDynamicComponents", () {
      app.createDynamicComponents();
      appRef.tick();
    });
  });
}

@Component(selector: "dummy", template: '''<div></div>''')
class DummyComponent {}

@Directive(selector: "[dummy-decorator]")
class DummyDirective {}

@Directive(selector: "dynamic-dummy")
class DynamicDummy {
  DynamicDummy(DynamicComponentLoader loader, ElementRef location) {
    loader.loadNextToLocation(DummyComponent, location);
  }
}

@Component(
    selector: "app",
    directives: const [
      NgIf,
      NgFor,
      DummyComponent,
      DummyDirective,
      DynamicDummy
    ],
    template: '''
    <div *ngIf="testingPlainComponents">
      <dummy *ngFor="#i of list"></dummy>
    </div>

    <div *ngIf="testingWithDirectives">
      <dummy dummy-decorator *ngFor="#i of list"></dummy>
    </div>

    <div *ngIf="testingDynamicComponents">
      <dynamic-dummy *ngFor="#i of list"></dynamic-dummy>
    </div>
  ''')
class AppComponent {
  List<dynamic> list;
  bool testingPlainComponents;
  bool testingWithDirectives;
  bool testingDynamicComponents;
  AppComponent() {
    this.reset();
  }
  void reset() {
    this.list = [];
    this.testingPlainComponents = false;
    this.testingWithDirectives = false;
    this.testingDynamicComponents = false;
  }

  void createPlainComponents() {
    this.list = testList;
    this.testingPlainComponents = true;
  }

  void createComponentsWithDirectives() {
    this.list = testList;
    this.testingWithDirectives = true;
  }

  void createDynamicComponents() {
    this.list = testList;
    this.testingDynamicComponents = true;
  }
}
