library benchmarks.src.di.di_benchmark;

import "package:angular2/core.dart"
    show Injectable, Injector, Key, bind, provide;
import "package:angular2/src/core/reflection/reflection.dart" show reflector;
import "package:angular2/src/core/reflection/reflection_capabilities.dart"
    show ReflectionCapabilities;
import "package:angular2/src/testing/benchmark_util.dart"
    show getIntParameter, bindAction, microBenchmark;
import "package:angular2/src/platform/browser/browser_adapter.dart"
    show BrowserDomAdapter;

var count = 0;
setupReflector() {
  reflector.reflectionCapabilities = new ReflectionCapabilities();
}

main() {
  BrowserDomAdapter.makeCurrent();
  var iterations = getIntParameter("iterations");
  // This benchmark does not use bootstrap and needs to create a reflector
  setupReflector();
  var bindings = [A, B, C, D, E];
  var injector = Injector.resolveAndCreate(bindings);
  var D_KEY = Key.get(D);
  var E_KEY = Key.get(E);
  var childInjector = injector
      .resolveAndCreateChild([]).resolveAndCreateChild([]).resolveAndCreateChild(
          []).resolveAndCreateChild([]).resolveAndCreateChild([]);
  var variousProviders = [
    A,
    provide(B, useClass: C),
    [
      D,
      [E]
    ],
    provide(F, useValue: 6)
  ];
  var variousProvidersResolved = Injector.resolve(variousProviders);
  getByToken() {
    for (var i = 0; i < iterations; ++i) {
      injector.get(D);
      injector.get(E);
    }
  }
  getByKey() {
    for (var i = 0; i < iterations; ++i) {
      injector.get(D_KEY);
      injector.get(E_KEY);
    }
  }
  getChild() {
    for (var i = 0; i < iterations; ++i) {
      childInjector.get(D);
      childInjector.get(E);
    }
  }
  instantiate() {
    for (var i = 0; i < iterations; ++i) {
      var child = injector.resolveAndCreateChild([E]);
      child.get(E);
    }
  }
  /**
   * Creates an injector with a variety of provider types.
   */
  createVariety() {
    for (var i = 0; i < iterations; ++i) {
      Injector.resolveAndCreate(variousProviders);
    }
  }
  /**
   * Same as [createVariety] but resolves providers ahead of time.
   */
  createVarietyResolved() {
    for (var i = 0; i < iterations; ++i) {
      Injector.fromResolvedProviders(variousProvidersResolved);
    }
  }
  bindAction(
      "#getByToken", () => microBenchmark("injectAvg", iterations, getByToken));
  bindAction(
      "#getByKey", () => microBenchmark("injectAvg", iterations, getByKey));
  bindAction(
      "#getChild", () => microBenchmark("injectAvg", iterations, getChild));
  bindAction("#instantiate",
      () => microBenchmark("injectAvg", iterations, instantiate));
  bindAction("#createVariety",
      () => microBenchmark("injectAvg", iterations, createVariety));
  bindAction("#createVarietyResolved",
      () => microBenchmark("injectAvg", iterations, createVarietyResolved));
}

@Injectable()
class A {
  A() {
    count++;
  }
}

@Injectable()
class B {
  B(A a) {
    count++;
  }
}

@Injectable()
class C {
  C(B b) {
    count++;
  }
}

@Injectable()
class D {
  D(C c, B b) {
    count++;
  }
}

@Injectable()
class E {
  E(D d, C c) {
    count++;
  }
}

@Injectable()
class F {
  F(E e, D d) {
    count++;
  }
}
